// -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
/*
 * Copyright (C) 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Brandon Schaefer <brandontschaefer@gmail.com>
 */

#ifndef BLOCKS_GRID_H
#define BLOCKS_GRID_H

#include "Cell.h"
#include "Grid.h"
#include "Piece.h"
#include "Timer.h"
#include <vector>

class SDLHandler;

namespace blocks
{

class BlocksGrid : public Grid
{
public:
  BlocksGrid(Point const& start, Point const& xy_cells, int cell_size, int spacing);

  void Update(float delta_time);
  void Draw(SDLHandler* sdl_handler);

  void ClearPieceRegion(Piece::Ptr const& piece);
  void UpdatePieceRegion(Piece::Ptr const& piece);

  bool CanPieceMoveDirection(Piece::Ptr& piece, Direction direction) const;
  bool CanPieceRotateClockwise(Piece::Ptr& piece) const;

  bool Animating() const;

  bool RowsNeedClearing();
  void RowClearingUpdate();
  void RowClearingDone();

  void AddNewPiece(Piece::Ptr const& piece);
  void RemovePiece(Piece::Ptr const& piece);

  void Lose();
  bool Lost();

  void Reset();

  sigc::signal<void> animation_done;

private:
  unsigned score_;
  bool lost_;
  unsigned fadding_index_;

  std::vector<int> rows_to_clear_;
  std::vector<Piece::Ptr> pieces_;

  Timer row_clearing_timer_;
};

}

#endif // BLOCKS_GRID_H
