// -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
/*
 * Copyright (C) 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Brandon Schaefer <brandontschaefer@gmail.com>
 */

#ifndef BLOCKS_GRID_CONTROLLER_H
#define BLOCKS_GRID_CONTROLLER_H

#include "BlocksGrid.h"
#include "Piece.h"
#include "PieceFactory.h"
#include "ScreenGrid.h"

class SDLHandler;

namespace blocks
{

class BlocksGridController
{
public:
  BlocksGridController(ScreenGrid const& screen, int width, int height);

  void Update();
  void UpdateTime(float delta_time);
  void Draw(SDLHandler* sdl_handler);

  void MoveCurrentPieceRight();
  void MoveCurrentPieceLeft();
  void MoveCurrentPieceDown();

  void HardDropCurrentPiece();

  void SwapNextPiece();

  void RotateCurrentPieceClockwise();

  bool Animating() const;

  void Reset();

private:
  void MovePiece(Direction direction);

  void DeepCopyCurrentPieceToGhostPiece();
  void UpdateGhostPiece();

  void PickNewCurrentPiece();

  bool IsCurrentPieceNearGhostPiece() const;

  int width_;
  int height_;

  ScreenGrid screen_;

  int center_x_;

  Point start_;
  Point nstart_;
  BlocksGrid grid_;
  BlocksGrid next_piece_grid_;

  PieceFactory piece_factory_;
  int current_piece_seed_;
  int next_piece_seed_;

  Piece::Ptr current_piece_;
  Piece::Ptr ghost_piece_;
  Piece::Ptr next_piece_;

  bool can_swap_;
  bool need_new_piece_;
  bool ghost_piece_active_;
  Direction dir_;
};

}

#endif // BLOCKS_GRID_CONTROLLER_H
