// -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
/*
 * Copyright (C) 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Brandon Schaefer <brandontschaefer@gmail.com>
 */

#include "Color.h"

#include <stdlib.h>

Color::Color(float r, float g, float b, float a)
{
  color_[0] = r;
  color_[1] = g;
  color_[2] = b;
  color_[3] = a;
}

Color::Color(GLfloat const color[4])
{
  color_[0] = color[0];
  color_[1] = color[1];
  color_[2] = color[2];
  color_[3] = color[3];
}

Color::Color(Color const& color)
{
  color_[0] = color.color_[0];
  color_[1] = color.color_[1];
  color_[2] = color.color_[2];
  color_[3] = color.color_[3];
}

bool Color::operator==(Color const& color) const
{
  if (color_[0] == color.color_[0] &&
      color_[1] == color.color_[1] &&
      color_[2] == color.color_[2] &&
      color_[3] == color.color_[3])
  {
    return true;
  }

  return false;
}

bool Color::operator!=(Color const& color) const
{
  return !(*this == color);
}

Color GetRandomColor()
{
  float r = (float)rand()/(float)RAND_MAX;
  float g = (float)rand()/(float)RAND_MAX;
  float b = (float)rand()/(float)RAND_MAX;
  float a = 1.0f;

  return Color(r,g,b,a);
}

Color SwapTwoColors(Color const& color1, Color const& color2)
{
  static Color last_color = color2;

  if (last_color == color1)
  {
    last_color = color2;
    return color2;
  }
  else
  {
    last_color = color1;
    return color1;
  }
}
