// -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
/*
 * Copyright (C) 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Brandon Schaefer <brandontschaefer@gmail.com>
 */

#ifndef GRID_H 
#define GRID_H

#include "Cell.h"
#include "Color.h"
#include "Piece.h"
#include <vector>

class SDLHandler;

namespace blocks
{

class Grid 
{
public:
  Grid(Point const& start, Point const& xy_cells, int cell_size, int spacing);

  void SetPhased(Point const& point);
  void UnsetPhased(Point const& point);
  void SetColor(Point const& point, Color const& color);

  void Draw(SDLHandler* sdl_handler);

protected:
  void InitGrid();
  bool InBounds(Point const& p) const;
  bool IsMarked(Point const& p) const;

  int GetXCells() const;
  int GetYCells() const;

private:
  int x_;
  int y_;
  int x_cells_;
  int y_cells_;
  int cell_size_;
  int spacing_;

  std::vector<std::vector<Cell> > grid_;
};

}

#endif // GRID_H
