// -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
/*
 * Copyright (C) 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Brandon Schaefer <brandontschaefer@gmail.com>
 */

#ifndef MAIN_LOOP_H
#define MAIN_LOOP_H

#include <EGL/egl.h>
#include <GLES2/gl2.h>

#include "Timer.h"
#include "SDLHandler.h"
#include "ScreenGrid.h"
#include "BlocksGridController.h"
#include "Button.h"

namespace blocks
{

class MainLoop
{
public:
  MainLoop();
  ~MainLoop();

  void Start();

private:
  bool InitWindowAndRenderer();
  bool InitShaders();

  void ResetBlocks();

  void Draw       (SDLHandler* sdl_handler);
  void Update     (SDLHandler* sdl_handler, float delta_time);
  void KeyEvent   (SDLHandler* sdl_handler, unsigned char key, int keysym, int state);
  void MouseEvent (SDLHandler* sdl_handler, int x, int y, unsigned int button, bool down);
  void MotionEvent(SDLHandler* sdl_handler, int x, int y, int dx, int dy, int timestamp);

  void UpdatePieceSpeed();
  void DonePieceSpeed();

  SDLHandler sdl_handler_;
  ScreenGrid screen_;

  BlocksGridController grid_controller_;

  Timer piece_speed_;

  Button pause_button_;
  Button reset_button_;

  float current_speed_;

  bool paused_;
  bool mouse_moved_;
};

}

#endif // MAIN_LOOP_H
