// -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
/*
 * Copyright (C) 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Brandon Schaefer <brandontschaefer@gmail.com>
 */

#ifndef PIECE_H
#define PIECE_H

#include "Color.h"
#include "Point.h"

#include <memory>
#include <vector>

namespace blocks
{

class Piece
{
public:
  typedef std::shared_ptr<Piece> Ptr;

  Piece();
  Piece(Piece& piece);

  std::vector<Point> GetPoints() const;
  Color GetColor() const;

  void SetAsGhostPiece();
  bool IsGhostPiece() const;

  void RemovePoint(Point const& point);

  void MoveDirection(Direction const& direction);
  void MoveOppositeDirection(Direction const& direction);

  void RotateClockwise();
  void UndoRotation();

  bool PointInPiece(Point const& point) const;

  void MoveDownPointsAboveRow(int row);

  virtual bool CanRotate() const = 0;

protected:
  Direction OppositeDirection(Direction const& direction) const;

  std::vector<Point> points_;
  Color color_;
  Point pivot_;

  bool is_ghost_;
};

}

#endif // PIECE_H
