// -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
/*
 * Copyright (C) 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Brandon Schaefer <brandontschaefer@gmail.com>
 */

#include "PieceFactory.h"

#include "IPiece.h"
#include "JPiece.h"
#include "LPiece.h"
#include "OPiece.h"
#include "SPiece.h"
#include "TPiece.h"
#include "ZPiece.h"

#include <stdlib.h>
#include <time.h>

namespace blocks
{

int const NUMBER_OF_PIECES = 7;

PieceFactory::PieceFactory()
{
}

std::vector<int> GetNewBagOfPieces()
{
  std::vector<int> bag_of_pieces;

  for (int i = 0; i < NUMBER_OF_PIECES - 1; ++i)
  {
    bag_of_pieces.push_back(i);
  }

  bool swap_s_to_z = rand() % 2;
  if (swap_s_to_z)
  {
    bag_of_pieces[SPIECE] = ZPIECE;
  }

  int tmp;
  int rand_index;
  for (int i = NUMBER_OF_PIECES - 2; i > 0; --i)
  {
    rand_index = rand() % i;

    tmp = bag_of_pieces[rand_index];
    bag_of_pieces[rand_index] = bag_of_pieces[i];
    bag_of_pieces[i] = tmp;
  }

  return bag_of_pieces;
}

int PieceFactory::GetRandomSeed()
{
  if (bag_of_pieces_.empty())
  {
    bag_of_pieces_ = GetNewBagOfPieces();
  }

  int seed = bag_of_pieces_.back();
  bag_of_pieces_.pop_back();

  return seed;
}

Piece::Ptr PieceFactory::GenerateRandomPiece(Point const& start, int seed) const
{
  int rand_index;
  if (seed >= 0)
    rand_index = seed % NUMBER_OF_PIECES;
  else
    rand_index = rand() % NUMBER_OF_PIECES;

  switch (rand_index)
  {
    case IPIECE:
      return std::make_shared<IPiece>(start);
    case JPIECE:
      return std::make_shared<JPiece>(start);
    case LPIECE:
      return std::make_shared<LPiece>(start);
    case OPIECE:
      return std::make_shared<OPiece>(start);
    case SPIECE:
      return std::make_shared<SPiece>(start);
    case TPIECE:
      return std::make_shared<TPiece>(start);
    case ZPIECE:
      return std::make_shared<ZPiece>(start);
    default:
      break;
  }

  return std::make_shared<OPiece>(start);
}

}
