// -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
/*
 * Copyright (C) 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Brandon Schaefer <brandontschaefer@gmail.com>
 */

#include "Point.h"

namespace blocks
{

Point::Point(int x, int y)
  : x_(x)
  , y_(y)
{
}

int Point::x() const
{
  return x_;
}

int Point::y() const
{
  return y_;
}

Point Point::Right() const
{
  return {x_ + 1, y_};
}

Point Point::Left() const
{
  return {x_ - 1, y_};
}

Point Point::Up() const
{
  return {x_, y_ + 1};
}

Point Point::Down() const
{
  return {x_, y_ - 1};
}

Point Point::MoveDirection(Direction const& direction)
{
  switch (direction)
  {
    case RIGHT:
      return Right();
    case LEFT:
      return Left();
    case UP:
      return Up();
    case DOWN:
      return Down();
    default:
      break;
  }

  return *this;
}

Point Point::operator+(Point const& point) const
{
  return Point(this->x_ + point.x(), this->y_ + point.y());
}

Point Point::operator-(Point const& point) const
{
  return Point(this->x_ - point.x(), this->y_ - point.y());
}

bool Point::operator==(Point const& point) const
{
  if (x_ == point.x_ && y_ == point.y_)
    return true;

  return false;
}

bool Point::operator!=(Point const& point) const
{
  return !(*this == point);
}

}
