// -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
/*
 * Copyright (C) 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Brandon Schaefer <brandontschaefer@gmail.com>
 */

#ifndef SDL_HANDLER_H 
#define SDL_HANDLER_H 

#include <SDL.h>

#include <sigc++/signal.h>

#include "ShadeLoader.h"

class SDLHandler 
{
public:
  SDLHandler(char const* title);
  ~SDLHandler();

  int Width() const;
  int Height() const;

  GLint GetGLProgram() const;

  void StartMainLoop();

  void ToggleFullscreen();

  void Exit();

  sigc::signal<void, SDLHandler*> draw;

  sigc::signal<void, SDLHandler*, float> update;

  sigc::signal<void, SDLHandler*, unsigned char, int, int> key_event;

  sigc::signal<void, SDLHandler*, int, int, unsigned int, bool> mouse_event;

  sigc::signal<void, SDLHandler*, int, int, int, int, int> motion_event;

private:
  SDL_Renderer* renderer_;
  SDL_Window* window_;
  SDL_GLContext context_;
  GLint program_;

  ShadeLoader shade_loader_;

  int width_;
  int height_;
  bool fullscreen_;
  bool done_;
};

#endif // SDL_HANDLER_H 
