// -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
/*
 * Copyright (C) 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Brandon Schaefer <brandontschaefer@gmail.com>
 */

#include "ScreenGrid.h"

ScreenGrid::ScreenGrid(int w, int h, int max_x_cells, int max_y_cells, int spacing)
  : width_(w)
  , height_(h)
  , spacing_(spacing)
{
  int cell_width  = w / (max_x_cells + spacing);
  int cell_height = h / (max_y_cells + spacing);

  cell_size_ = std::min(cell_width, cell_height);
}

int ScreenGrid::GetCenterX(int x_cells) const
{
  int half_width            = width_ / 2;
  int cell_size_space       = cell_size_;
  int half_cell_size_amount = x_cells / 2 + 2;

  return half_width - cell_size_space * half_cell_size_amount;
}

int ScreenGrid::GetX(int x) const
{
  return x * (cell_size_ + spacing_);
}

int ScreenGrid::GetY(int y) const
{
  return y * (cell_size_ + spacing_);
}

int ScreenGrid::CellSize() const
{
  return cell_size_;
}
